==> framework-1.1.16-sources.jar: com/liecoder/framework/ktx/ShapeKtx.kt <==
package com.liecoder.framework.ktx

import android.graphics.Outline
import android.view.View
import android.view.ViewOutlineProvider

/**
 * 设置视图的圆角。
 *
 * 此扩展函数为任意视图提供了设置圆角的功能。如果指定的圆角大小为0，则不应用圆角效果；否则，将视图的轮廓设置为圆角矩形。
 *
 * @param radius 圆角的尺寸，以像素为单位。如果该值为0，则视图不会有圆角效果。
 * @return 返回自身，允许链式调用。
 */
fun <T : View> T.radius(radius: Int): T {
    if (radius == 0) {
        clipToOutline = false
    } else {
        clipToOutline = true
        outlineProvider = object : ViewOutlineProvider() {
            override fun getOutline(
                view: View?, outLine: Outline?
            ) {
                outLine?.setRoundRect(0, 0, view?.width ?: 0, view?.height ?: 0, radius.dpFloat)
            }
        }
    }
    return this
}


/**
 * 设置视图顶部圆角。
 *
 * 此扩展函数为任意视图提供了设置顶部圆角的功能。如果指定的圆角大小为0，则不应用圆角效果；否则，将视图顶部的轮廓设置为圆角。
 *
 * @param radius 圆角的尺寸，以像素为单位。如果该值为0，则视图顶部不会有圆角效果。
 * @return 返回自身，允许链式调用。
 */
fun <T : View> T.topRadius(radius: Int): T {
    if (radius == 0) {
        clipToOutline = false
    } else {
        clipToOutline = true
        outlineProvider = object : ViewOutlineProvider() {
            override fun getOutline(
                view: View?, outline: Outline?
            ) {
                outline?.setRoundRect(
                    0, 0, view?.width ?: 0, (view?.height ?: 0) + radius.dpInt, radius.dpFloat
                )
            }
        }
    }
    return this
}

/**
 * 设置视图底部圆角。
 *
 * 此扩展函数为任意视图提供了设置底部圆角的功能。如果指定的圆角大小为0，则不应用圆角效果；否则，将视图底部的轮廓设置为圆角。
 *
 * @param radius 圆角的尺寸，以像素为单位。如果该值为0，则视图底部不会有圆角效果。
 * @return 返回自身，允许链式调用。
 */
fun <T : View> T.bottomRadius(radius: Int): T {
    if (radius == 0) {
        clipToOutline = false
    } else {
        clipToOutline = true
        outlineProvider = object : ViewOutlineProvider() {
            override fun getOutline(
                view: View?, outLine: Outline?
            ) {
                outLine?.setRoundRect(
                    0, -radius.dpInt, view?.width ?: 0, view?.height ?: 0, radius.dpFloat
                )
            }
        }
    }
    return this
}

==> framework-2.2.1-sources.jar: com/liecoder/framework/ktx/ShapeKtx.kt <==
package com.liecoder.framework.ktx

import android.animation.Animator
import android.animation.ValueAnimator
import android.graphics.Outline
import android.view.View
import android.view.ViewGroup
import android.view.ViewOutlineProvider

/**
 * 设置视图的圆角。
 *
 * 此扩展函数为任意视图提供了设置圆角的功能。如果指定的圆角大小为0，则不应用圆角效果；否则，将视图的轮廓设置为圆角矩形。
 *
 * @param radius 圆角的尺寸，以像素为单位。如果该值为0，则视图不会有圆角效果。
 * @return 返回自身，允许链式调用。
 */
fun <T : View> T.radius(radius: Int): T {
    if (radius == 0) {
        clipToOutline = false
    } else {
        clipToOutline = true
        outlineProvider = object : ViewOutlineProvider() {
            override fun getOutline(
                view: View?, outLine: Outline?
            ) {
                outLine?.setRoundRect(0, 0, view?.width ?: 0, view?.height ?: 0, radius.dpFloat)
            }
        }
    }
    return this
}


/**
 * 设置视图顶部圆角。
 *
 * 此扩展函数为任意视图提供了设置顶部圆角的功能。如果指定的圆角大小为0，则不应用圆角效果；否则，将视图顶部的轮廓设置为圆角。
 *
 * @param radius 圆角的尺寸，以像素为单位。如果该值为0，则视图顶部不会有圆角效果。
 * @return 返回自身，允许链式调用。
 */
fun <T : View> T.topRadius(radius: Int): T {
    if (radius == 0) {
        clipToOutline = false
    } else {
        clipToOutline = true
        outlineProvider = object : ViewOutlineProvider() {
            override fun getOutline(
                view: View?, outline: Outline?
            ) {
                outline?.setRoundRect(
                    0, 0, view?.width ?: 0, (view?.height ?: 0) + radius.dpInt, radius.dpFloat
                )
            }
        }
    }
    return this
}

/**
 * 设置视图底部圆角。
 *
 * 此扩展函数为任意视图提供了设置底部圆角的功能。如果指定的圆角大小为0，则不应用圆角效果；否则，将视图底部的轮廓设置为圆角。
 *
 * @param radius 圆角的尺寸，以像素为单位。如果该值为0，则视图底部不会有圆角效果。
 * @return 返回自身，允许链式调用。
 */
fun <T : View> T.bottomRadius(radius: Int): T {
    if (radius == 0) {
        clipToOutline = false
    } else {
        clipToOutline = true
        outlineProvider = object : ViewOutlineProvider() {
            override fun getOutline(
                view: View?, outLine: Outline?
            ) {
                outLine?.setRoundRect(
                    0, -radius.dpInt, view?.width ?: 0, view?.height ?: 0, radius.dpFloat
                )
            }
        }
    }
    return this
}



/**
 * 设置高度，带有过渡动画
 * @param targetValue 目标高度
 * @param duration 时长
 * @param action 可选行为
 */
fun View.animateHeight(
    targetValue: Int,
    duration: Long = 400,
    listener: Animator.AnimatorListener? = null,
    action: ((Float) -> Unit)? = null
) {
    post {
        ValueAnimator
            .ofInt(height, targetValue)
            .apply {
                addUpdateListener {
                    height(it.animatedValue as Int)
                    action?.invoke((it.animatedFraction))
                }
                if (listener != null) addListener(listener)
                setDuration(duration)
                start()
            }
    }
}

/**
 * 设置View的高度
 */
fun View.height(height: Int): View {
    val params = layoutParams ?: ViewGroup.LayoutParams(
        ViewGroup.LayoutParams.MATCH_PARENT,
        ViewGroup.LayoutParams.WRAP_CONTENT)
    params.height = height
    layoutParams = params
    return this
}
